@@ -11,15 +11,19 @@
 
 namespace Predis;
 
-use Predis\Commands\ICommand;
-use Predis\Options\IClientOptions;
-use Predis\Network\IConnection;
-use Predis\Network\IConnectionSingle;
-use Predis\Profiles\IServerProfile;
-use Predis\Options\ClientOptions;
-use Predis\Profiles\ServerProfile;
-use Predis\PubSub\PubSubContext;
+use Predis\Command\CommandInterface;
+use Predis\Command\ScriptedCommand;
+use Predis\Connection\AggregatedConnectionInterface;
+use Predis\Connection\ConnectionInterface;
+use Predis\Connection\ConnectionFactory;
+use Predis\Connection\ConnectionFactoryInterface;
+use Predis\Monitor\MonitorContext;
+use Predis\Option\ClientOptions;
+use Predis\Option\ClientOptionsInterface;
 use Predis\Pipeline\PipelineContext;
+use Predis\Profile\ServerProfile;
+use Predis\Profile\ServerProfileInterface;
+use Predis\PubSub\PubSubContext;
 use Predis\Transaction\MultiExecContext;
 
 /**
@@ -27,9 +31,9 @@ use Predis\Transaction\MultiExecContext;
  *
  * @author Daniele Alessandri <suppakilla@gmail.com>
  */
-class Client
+class Client implements ClientInterface
 {
-    const VERSION = '0.7.3';
+    const VERSION = '0.8.2';
 
     private $options;
     private $profile;
@@ -44,17 +48,16 @@ class Client
      */
     public function __construct($parameters = null, $options = null)
     {
-        $options = $this->filterOptions($options);
-        $this->options = $options;
-        $this->profile = $options->profile;
-        $this->connections = $options->connections;
+        $this->options = $this->filterOptions($options);
+        $this->profile = $this->options->profile;
+        $this->connections = $this->options->connections;
         $this->connection = $this->initializeConnection($parameters);
     }
 
     /**
-     * Creates an instance of Predis\Options\ClientOptions from various types of
-     * arguments (string, array, Predis\Profiles\ServerProfile) or returns the
-     * passed object if it is an instance of Predis\Options\ClientOptions.
+     * Creates an instance of Predis\Option\ClientOptions from various types of
+     * arguments (string, array, Predis\Profile\ServerProfile) or returns the
+     * passed object if it is an instance of Predis\Option\ClientOptions.
      *
      * @param mixed $options Client options.
      * @return ClientOptions
@@ -64,15 +67,14 @@ class Client
         if ($options === null) {
             return new ClientOptions();
         }
+
         if (is_array($options)) {
             return new ClientOptions($options);
         }
-        if ($options instanceof IClientOptions) {
+
+        if ($options instanceof ClientOptionsInterface) {
             return $options;
         }
-        if ($options instanceof IServerProfile || is_string($options)) {
-            return new ClientOptions(array('profile' => $options));
-        }
 
         throw new \InvalidArgumentException("Invalid type for client options");
     }
@@ -80,33 +82,29 @@ class Client
     /**
      * Initializes one or multiple connection (cluster) objects from various
      * types of arguments (string, array) or returns the passed object if it
-     * implements the Predis\Network\IConnection interface.
+     * implements Predis\Connection\ConnectionInterface.
      *
      * @param mixed $parameters Connection parameters or instance.
-     * @return IConnection
+     * @return ConnectionInterface
      */
     protected function initializeConnection($parameters)
     {
-        if ($parameters instanceof IConnection) {
+        if ($parameters instanceof ConnectionInterface) {
             return $parameters;
         }
 
         if (is_array($parameters) && isset($parameters[0])) {
             $replication = isset($this->options->replication) && $this->options->replication;
-
             $connection = $this->options->{$replication ? 'replication' : 'cluster'};
-            $initializer = $replication ? 'createReplication' : 'createCluster';
 
-            return $this->connections->$initializer($connection, $parameters, $this->profile);
+            return $this->connections->createAggregated($connection, $parameters);
         }
 
-        return $this->connections->create($parameters, $this->profile);
+        return $this->connections->create($parameters);
     }
 
     /**
-     * Returns the server profile used by the client.
-     *
-     * @return IServerProfile
+     * {@inheritdoc}
      */
     public function getProfile()
     {
@@ -114,9 +112,7 @@ class Client
     }
 
     /**
-     * Returns the client options specified upon initialization.
-     *
-     * @return ClientOptions
+     * {@inheritdoc}
      */
     public function getOptions()
     {
@@ -126,7 +122,7 @@ class Client
     /**
      * Returns the connection factory object used by the client.
      *
-     * @return IConnectionFactory
+     * @return ConnectionFactoryInterface
      */
     public function getConnectionFactory()
     {
@@ -140,13 +136,13 @@ class Client
      *
      * @return Client
      */
-    public function getClientFor($connectionAlias)
+    public function getClientFor($connectionID)
     {
-        if (($connection = $this->getConnection($connectionAlias)) === null) {
-            throw new \InvalidArgumentException("Invalid connection alias: '$connectionAlias'");
+        if (($connection = $this->getConnectionById($connectionID)) === null) {
+            throw new \InvalidArgumentException("Invalid connection ID: '$connectionID'");
         }
 
-        return new Client($connection, $this->options);
+        return new static($connection, $this->options);
     }
 
     /**
@@ -187,27 +183,30 @@ class Client
     }
 
     /**
-     * Returns the underlying connection instance or, when connected to a cluster,
-     * one of the connection instances identified by its alias.
-     *
-     * @param string $id The alias of a connection when connected to a cluster.
-     * @return IConnection
+     * {@inheritdoc}
      */
-    public function getConnection($id = null)
+    public function getConnection()
     {
-        if (isset($id)) {
-            if (!Helpers::isAggregated($this->connection)) {
-                $message = 'Retrieving connections by alias is supported only with aggregated connections (cluster or replication)';
-                throw new NotSupportedException($message);
-            }
-            return $this->connection->getConnectionById($id);
-        }
-
         return $this->connection;
     }
 
     /**
-     * Dinamically invokes a Redis command with the specified arguments.
+     * Retrieves a single connection out of an aggregated connections instance.
+     *
+     * @param string $connectionId Index or alias of the connection.
+     * @return SingleConnectionInterface
+     */
+    public function getConnectionById($connectionId)
+    {
+        if (!$this->connection instanceof AggregatedConnectionInterface) {
+            throw new NotSupportedException('Retrieving connections by ID is supported only when using aggregated connections');
+        }
+
+        return $this->connection->getConnectionById($connectionId);
+    }
+
+    /**
+     * Dynamically invokes a Redis command with the specified arguments.
      *
      * @param string $method The name of a Redis command.
      * @param array $arguments The arguments for the command.
@@ -216,15 +215,21 @@ class Client
     public function __call($method, $arguments)
     {
         $command = $this->profile->createCommand($method, $arguments);
-        return $this->connection->executeCommand($command);
+        $response = $this->connection->executeCommand($command);
+
+        if ($response instanceof ResponseObjectInterface) {
+            if ($response instanceof ResponseErrorInterface) {
+                $response = $this->onResponseError($command, $response);
+            }
+
+            return $response;
+        }
+
+        return $command->parseResponse($response);
     }
 
     /**
-     * Creates a new instance of the specified Redis command.
-     *
-     * @param string $method The name of a Redis command.
-     * @param array $arguments The arguments for the command.
-     * @return ICommand
+     * {@inheritdoc}
      */
     public function createCommand($method, $arguments = array())
     {
@@ -232,35 +237,50 @@ class Client
     }
 
     /**
-     * Executes the specified Redis command.
-     *
-     * @param ICommand $command A Redis command.
-     * @return mixed
+     * {@inheritdoc}
      */
-    public function executeCommand(ICommand $command)
+    public function executeCommand(CommandInterface $command)
     {
-        return $this->connection->executeCommand($command);
+        $response = $this->connection->executeCommand($command);
+
+        if ($response instanceof ResponseObjectInterface) {
+            if ($response instanceof ResponseErrorInterface) {
+                $response = $this->onResponseError($command, $response);
+            }
+
+            return $response;
+        }
+
+        return $command->parseResponse($response);
     }
 
     /**
-     * Executes the specified Redis command on all the nodes of a cluster.
+     * Handles -ERR responses returned by Redis.
      *
-     * @param ICommand $command A Redis command.
-     * @return array
+     * @param CommandInterface $command The command that generated the error.
+     * @param ResponseErrorInterface $response The error response instance.
+     * @return mixed
      */
-    public function executeCommandOnShards(ICommand $command)
+    protected function onResponseError(CommandInterface $command, ResponseErrorInterface $response)
     {
-        if (Helpers::isCluster($this->connection)) {
-            $replies = array();
+        if ($command instanceof ScriptedCommand && $response->getErrorType() === 'NOSCRIPT') {
+            $eval = $this->createCommand('eval');
+            $eval->setRawArguments($command->getEvalArguments());
 
-            foreach ($this->connection as $connection) {
-                $replies[] = $connection->executeCommand($command);
+            $response = $this->executeCommand($eval);
+
+            if (false === $response instanceof ResponseObjectInterface) {
+                $response = $command->parseResponse($response);
             }
 
-            return $replies;
+            return $response;
         }
 
-        return array($this->connection->executeCommand($command));
+        if ($this->options->exceptions === true) {
+            throw new ServerException($response->getMessage());
+        }
+
+        return $response;
     }
 
     /**
@@ -312,8 +332,16 @@ class Client
      */
     protected function initPipeline(Array $options = null, $callable = null)
     {
-        $pipeline = new PipelineContext($this, $options);
-        return $this->pipelineExecute($pipeline, $callable);
+        $executor = isset($options['executor']) ? $options['executor'] : null;
+
+        if (is_callable($executor)) {
+            $executor = call_user_func($executor, $this, $options);
+        }
+
+        $pipeline = new PipelineContext($this, $executor);
+        $replies  = $this->pipelineExecute($pipeline, $callable);
+
+        return $replies;
     }
 
     /**
